@@ -2,13 +2,14 @@ na.roughfix <- function(object, ...)
   UseMethod("na.roughfix")
 
 na.roughfix.data.frame <- function(object, ...) {
-  n <- length(object)
-  vars <- seq(length = n)
+##  n <- length(object)
+##  vars <- seq(length = n)
   isfac <- sapply(object, is.factor)
   isnum <- sapply(object, is.numeric)
+  hasNA <- which(sapply(object, function(x) any(is.na(x))))
   if (any(!(isfac | isnum)))
-      stop("na.roughfix only works for numeric or factor,")
-  for (j in vars) {
+      stop("na.roughfix only works for numeric or factor")
+  for (j in hasNA) {
     if (isfac[j]) {
       freq <- table(object[[j]])
       xmode <- names(freq)[max.col(rbind(freq))]
@@ -26,12 +27,17 @@ na.roughfix.default <- function(object, ...) {
     return(object)
   d <- dim(object)
   if (length(d) > 2) 
-    return(object)
+    stop("can't handle objects with more than two dimensions")
   if (all(!is.na(object)))
     return(object)
   if (!is.numeric(object))
     stop("roughfix can only deal with numeric data.")
-  for (j in seq(along=ncol(object))) 
-    object[is.na(object[, j]), j] <- median(object[, j], na.rm=TRUE)
+  if (d == 2) {
+      hasNA <- which(apply(object, 2, function(x) any(is.na(x))))
+      for (j in hasNA) 
+          object[is.na(object[, j]), j] <- median(object[, j], na.rm=TRUE)
+  } else {
+      object[is.na(object)] <- median(object, na.rm=TRUE)
+  }
   object
 }
